### ===================================================================================================================
###                                                       VIIA
###                                                  Objectnr: XXXX
###                                             Structural engineer: Name
### ===================================================================================================================
# TEMPLATE NLTH v2.0
# Copyright VIIA

# Note: The mainscript template is divided into several sections following the steps in NLTH protocol. Functions that
# can be used in each step are placed under the corresponding section. Some functions can only be used under certain
# conditions, please check the docstring before calling them.

# =====================================================================================================================
#                                                    Initial startup
# =====================================================================================================================


# Import the viiaPackage
from viiapackage import *

from rhdhv_fem.fem_math import fem_unit_vector_2_points

# Bools for script control
run_analysis = True

# Initiate the session
project = viia_create_project(project_name='conn_workshop-model', analysis_type='NLTH', version_nr=1)

# Load the model in the json that was created with the model script (model sub folder)
project.viia_read_dump('conn_workshop-model.json', False)

# Assign all shapes to variables
# Ground floor
fstrip1 = project.get_shape(collection_name='fstrips', _id=1)
fstrip2 = project.get_shape(collection_name='fstrips', _id=18)

wall1 = project.get_shape(collection_name='walls', _id=1)
wall2 = project.get_shape(collection_name='walls', _id=2)
wall3 = project.get_shape(collection_name='walls', _id=3)
wall4 = project.get_shape(collection_name='walls', _id=4)

# First floor
wall5 = project.get_shape(collection_name='walls', _id=5)
wall6 = project.get_shape(collection_name='walls', _id=6)
wall7 = project.get_shape(collection_name='walls', _id=7)
wall8 = project.get_shape(collection_name='walls', _id=8)

floor1 = project.get_shape(collection_name='floors', _id=1)

beam1 = project.get_shape(collection_name='beams', _id=1)
beam3 = project.get_shape(collection_name='beams', _id=3)
beam4 = project.get_shape(collection_name='beams', _id=4)
beam5 = project.get_shape(collection_name='beams', _id=5)
beam6 = project.get_shape(collection_name='beams', _id=6)

column1 = project.get_shape(collection_name='columns', _id=1)

# Second floor
wall9 = project.get_shape(collection_name='walls', _id=9)
wall10 = project.get_shape(collection_name='walls', _id=10)
wall11 = project.get_shape(collection_name='walls', _id=11)
wall12 = project.get_shape(collection_name='walls', _id=12)

beam8 = project.get_shape(collection_name='beams', _id=8)
beam9 = project.get_shape(collection_name='beams', _id=9)
beam10 = project.get_shape(collection_name='beams', _id=10)
beam11 = project.get_shape(collection_name='beams', _id=11)
beam12 = project.get_shape(collection_name='beams', _id=12)

# Third floor)
wall13 = project.get_shape(collection_name='walls', _id=13)
wall14 = project.get_shape(collection_name='walls', _id=14)
wall15 = project.get_shape(collection_name='walls', _id=15)
wall16 = project.get_shape(collection_name='walls', _id=16)

floor20 = project.get_shape(collection_name='floors', _id=20)

# Roof
roof21 = project.get_shape(collection_name='roofs', _id=21)
roof22 = project.get_shape(collection_name='roofs', _id=22)
roof23 = project.get_shape(collection_name='roofs', _id=23)
roof24 = project.get_shape(collection_name='roofs', _id=24)

beam13 = project.get_shape(collection_name='beams', _id=13)
beam14 = project.get_shape(collection_name='beams', _id=14)

# Define exercise solutions to view
# Note: The following solutions won't work together:
#   1 and 6
#   1 and 8
#   6 and 8
exercises = [1, 2, 3, 4, 5, 7]

# EXERCISE SOLUTIONS
if 6 in exercises:
    walls = [wall13, wall14, wall15, wall16]
    roofs = [roof21, roof22, roof23, roof24]
    for wall in walls:
        for roof in roofs:
            connecting_shapes = wall.get_connecting_shapes()
            if roof in connecting_shapes:
                project.viia_create_connection(detail='D5.01B',
                                               source_name=wall,
                                               target_name=roof)

if 7 in exercises:
    # Solution 1 (best)
    project.viia_create_connection(detail='D1.01',
                                   source_name=floor20,
                                   target_name=wall10,
                                   switch=True)

    # Solution 2
    project.viia_create_connection(detail='D1.01',
                                   source_name=floor20,
                                   target_name=wall12,
                                   flip=True)

if 8 in exercises:
    wall = wall14
    roof = roof21
    local_y_axis = fem_unit_vector_2_points([4.3075, 0.105, 7.29], [4.3075, 2.8725, 9.675])
    project.viia_create_connection(detail='D5.01B',
                                   source_name=wall,
                                   target_name=roof,
                                   local_y_axis=local_y_axis)

auto_interfaces_kwargs = {}
auto_hinges_kwargs = {}

custom_functions = []

if 1 in exercises:
    walls = [wall13, wall14, wall15, wall16]
    roofs = [roof21, roof22, roof23, roof24]
    overrides = [[wall.name, roof.name,"D5.01B"] for wall, roof in zip(walls,roofs)]
    auto_interfaces_kwargs["override_connections"] = overrides

if 2 in exercises:
    disable = [[beam4.name, floor1.name]]
    auto_interfaces_kwargs["disable_connections"] = disable

    def custom_interface():
        createConnection( "N1-LIJN-HINGE-1", "HINGE", SHAPEEDGE, SHAPEEDGE )
        setParameter( GEOMETRYCONNECTION, "N1-LIJN-HINGE-1", "MODE", "CLOSED" )
        setParameter( GEOMETRYCONNECTION, "N1-LIJN-HINGE-1", "DOF/AXES", [ 1, 2 ] )
        setParameter( GEOMETRYCONNECTION, "N1-LIJN-HINGE-1", "DOF/TRANSL", [ 1, 1, 1 ] )
        setParameter( GEOMETRYCONNECTION, "N1-LIJN-HINGE-1", "DOF/ROTATI", [ 0, 0, 0 ] )
        attachTo( GEOMETRYCONNECTION, "N1-LIJN-HINGE-1", "SOURCE", "N1-BALKEN-STAAL-HEA220-4", [[ 4.3075, 4.915, 0.135 ]] )
        attachTo( GEOMETRYCONNECTION, "N1-LIJN-HINGE-1", "TARGET", "N1-VLOEREN-LIN-HBV-PLANKEN-0.018-0.080-0.200-0.770-1", [[ 4.3075, 4.915, 0.135 ]] )

        createConnection( "N1-LIJN-HINGE-2", "HINGE", SHAPEEDGE, SHAPEEDGE )
        setParameter( GEOMETRYCONNECTION, "N1-LIJN-HINGE-2", "MODE", "CLOSED" )
        setParameter( GEOMETRYCONNECTION, "N1-LIJN-HINGE-2", "DOF/AXES", [ 1, 2 ] )
        setParameter( GEOMETRYCONNECTION, "N1-LIJN-HINGE-2", "DOF/TRANSL", [ 1, 1, 1 ] )
        setParameter( GEOMETRYCONNECTION, "N1-LIJN-HINGE-2", "DOF/ROTATI", [ 0, 0, 0 ] )
        attachTo( GEOMETRYCONNECTION, "N1-LIJN-HINGE-2", "SOURCE", "N1-BALKEN-STAAL-HEA220-4", [[ 4.3075, 2.1475, 0.135 ]] )
        attachTo( GEOMETRYCONNECTION, "N1-LIJN-HINGE-2", "TARGET", "N1-VLOEREN-LIN-HBV-PLANKEN-0.018-0.080-0.200-0.770-1", [[ 4.3075, 2.1475, 0.135 ]] )
    custom_functions.append(custom_interface)

if 3 in exercises:
    auto_interfaces_kwargs["replace_interfaces"] = {"D2.01":["D2.01",{"is_linear": True}]}

if 4 in exercises:
    auto_hinges_kwargs["override_hinges"] = [["N1-KOLOMEN-LIN-HOUT-200x200-1","N1-WANDEN-MW-KLEI<1945-350-5","H_RRF"]]

if 5 in exercises:
    auto_hinges_kwargs["disable_nodes"] = [project.find_node(point=[4.30750,2.87250,9.67500])]

project.viia_auto_interfaces(**auto_interfaces_kwargs)
project.viia_auto_hinges(**auto_hinges_kwargs)

# Create loads and boundary conditions
project.viia_create_load_combination_static()
project.viia_create_supports('FixedBase')

# Create model in diana
if project.rhdhvDIANA.run_diana:
    project.viia_create_model()
    for function in custom_functions:
        function()

# For fixed base analysis set the materials (temporarily to linear)
# project.viia_linear_properties()

if project.rhdhvDIANA.run_diana and run_analysis:

    project.viia_create_datfile()

    # Linear elastic static calculation (step A1 protocol)
    project.viia_analysis(analysis_nr='A1', run=run_analysis)

